@@ -1,21 +1,24 @@
-# Copyright (C) 2021, Mindee.
+# Copyright (C) 2021-2023, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
 import os
 import shutil
 from pathlib import Path
 from typing import Any, Callable, List, Optional, Tuple, Union
 
+import numpy as np
+
 from doctr.io.image import get_img_shape
 from doctr.utils.data import download_from_url
 
-__all__ = ['_AbstractDataset', '_VisionDataset']
+from ...models.utils import _copy_tensor
+
+__all__ = ["_AbstractDataset", "_VisionDataset"]
 
 
 class _AbstractDataset:
-
     data: List[Any] = []
     _pre_transforms: Optional[Callable[[Any, Any], Tuple[Any, Any]]] = None
 
@@ -26,9 +29,8 @@ class _AbstractDataset:
         sample_transforms: Optional[Callable[[Any, Any], Tuple[Any, Any]]] = None,
         pre_transforms: Optional[Callable[[Any, Any], Tuple[Any, Any]]] = None,
     ) -> None:
-
         if not Path(root).is_dir():
-            raise ValueError(f'expected a path to a reachable folder: {root}')
+            raise ValueError(f"expected a path to a reachable folder: {root}")
 
         self.root = root
         self.img_transforms = img_transforms
@@ -42,11 +44,7 @@ class _AbstractDataset:
     def _read_sample(self, index: int) -> Tuple[Any, Any]:
         raise NotImplementedError
 
-    def __getitem__(
-        self,
-        index: int
-    ) -> Tuple[Any, Any]:
-
+    def __getitem__(self, index: int) -> Tuple[Any, Any]:
         # Read image
         img, target = self._read_sample(index)
         # Pre-transforms (format conversion at run-time etc.)
@@ -55,10 +53,21 @@ class _AbstractDataset:
 
         if self.img_transforms is not None:
             # typing issue cf. https://github.com/python/mypy/issues/5485
-            img = self.img_transforms(img)  # type: ignore[call-arg]
+            img = self.img_transforms(img)
 
         if self.sample_transforms is not None:
-            img, target = self.sample_transforms(img, target)
+            # Conditions to assess it is detection model with multiple classes and avoid confusion with other tasks.
+            if (
+                isinstance(target, dict)
+                and all([isinstance(item, np.ndarray) for item in target.values()])
+                and set(target.keys()) != {"boxes", "labels"}  # avoid confusion with obj detection target
+            ):
+                img_transformed = _copy_tensor(img)
+                for class_name, bboxes in target.items():
+                    img_transformed, target[class_name] = self.sample_transforms(img, bboxes)
+                img = img_transformed
+            else:
+                img, target = self.sample_transforms(img, target)
 
         return img, target
 
@@ -95,9 +104,13 @@ class _VisionDataset(_AbstractDataset):
         cache_subdir: Optional[str] = None,
         **kwargs: Any,
     ) -> None:
+        cache_dir = (
+            str(os.environ.get("DOCTR_CACHE_DIR", os.path.join(os.path.expanduser("~"), ".cache", "doctr")))
+            if cache_dir is None
+            else cache_dir
+        )
 
-        cache_dir = os.path.join(os.path.expanduser('~'), '.cache', 'doctr') if cache_dir is None else cache_dir
-        cache_subdir = 'datasets' if cache_subdir is None else cache_subdir
+        cache_subdir = "datasets" if cache_subdir is None else cache_subdir
 
         file_name = file_name if isinstance(file_name, str) else os.path.basename(url)
         # Download the file if not present
