@@ -1,9 +1,10 @@
-# Copyright (C) 2021-2022, Mindee.
+# Copyright (C) 2021-2023, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
 
+from copy import deepcopy
 from typing import Any, Callable, Dict, List, Optional, Tuple
 
 from torch import nn
@@ -17,44 +18,44 @@ from doctr.datasets import VOCABS
 
 from ...utils import conv_sequence_pt, load_pretrained_params
 
-__all__ = ['ResNet', 'resnet18', 'resnet31', 'resnet34', 'resnet50', 'resnet34_wide', 'resnet_stage']
+__all__ = ["ResNet", "resnet18", "resnet31", "resnet34", "resnet50", "resnet34_wide", "resnet_stage"]
 
 
 default_cfgs: Dict[str, Dict[str, Any]] = {
-    'resnet18': {
-        'mean': (0.694, 0.695, 0.693),
-        'std': (0.299, 0.296, 0.301),
-        'input_shape': (3, 32, 32),
-        'classes': list(VOCABS['french']),
-        'url': 'https://github.com/mindee/doctr/releases/download/v0.4.1/resnet18-244bf390.pt',
+    "resnet18": {
+        "mean": (0.694, 0.695, 0.693),
+        "std": (0.299, 0.296, 0.301),
+        "input_shape": (3, 32, 32),
+        "classes": list(VOCABS["french"]),
+        "url": "https://doctr-static.mindee.com/models?id=v0.4.1/resnet18-244bf390.pt&src=0",
     },
-    'resnet31': {
-        'mean': (0.694, 0.695, 0.693),
-        'std': (0.299, 0.296, 0.301),
-        'input_shape': (3, 32, 32),
-        'classes': list(VOCABS['french']),
-        'url': 'https://github.com/mindee/doctr/releases/download/v0.4.1/resnet31-1056cc5c.pt',
+    "resnet31": {
+        "mean": (0.694, 0.695, 0.693),
+        "std": (0.299, 0.296, 0.301),
+        "input_shape": (3, 32, 32),
+        "classes": list(VOCABS["french"]),
+        "url": "https://doctr-static.mindee.com/models?id=v0.4.1/resnet31-1056cc5c.pt&src=0",
     },
-    'resnet34': {
-        'mean': (0.694, 0.695, 0.693),
-        'std': (0.299, 0.296, 0.301),
-        'input_shape': (3, 32, 32),
-        'classes': list(VOCABS['french']),
-        'url': 'https://github.com/mindee/doctr/releases/download/v0.5.0/resnet34-bd8725db.pt',
+    "resnet34": {
+        "mean": (0.694, 0.695, 0.693),
+        "std": (0.299, 0.296, 0.301),
+        "input_shape": (3, 32, 32),
+        "classes": list(VOCABS["french"]),
+        "url": "https://doctr-static.mindee.com/models?id=v0.5.0/resnet34-bd8725db.pt&src=0",
     },
-    'resnet50': {
-        'mean': (0.694, 0.695, 0.693),
-        'std': (0.299, 0.296, 0.301),
-        'input_shape': (3, 32, 32),
-        'classes': list(VOCABS['french']),
-        'url': 'https://github.com/mindee/doctr/releases/download/v0.5.0/resnet50-1a6c155e.pt',
+    "resnet50": {
+        "mean": (0.694, 0.695, 0.693),
+        "std": (0.299, 0.296, 0.301),
+        "input_shape": (3, 32, 32),
+        "classes": list(VOCABS["french"]),
+        "url": "https://doctr-static.mindee.com/models?id=v0.5.0/resnet50-1a6c155e.pt&src=0",
     },
-    'resnet34_wide': {
-        'mean': (0.694, 0.695, 0.693),
-        'std': (0.299, 0.296, 0.301),
-        'input_shape': (3, 32, 32),
-        'classes': list(VOCABS['french']),
-        'url': None,
+    "resnet34_wide": {
+        "mean": (0.694, 0.695, 0.693),
+        "std": (0.299, 0.296, 0.301),
+        "input_shape": (3, 32, 32),
+        "classes": list(VOCABS["french"]),
+        "url": "https://doctr-static.mindee.com/models?id=v0.6.0/resnet34_wide-b4b3e39e.pt&src=0",
     },
 }
 
@@ -105,8 +106,8 @@ class ResNet(nn.Sequential):
         attn_module: Optional[Callable[[int], nn.Module]] = None,
         include_top: bool = True,
         num_classes: int = 1000,
+        cfg: Optional[Dict[str, Any]] = None,
     ) -> None:
-
         _layers: List[nn.Module]
         if origin_stem:
             _layers = [
@@ -120,8 +121,9 @@ class ResNet(nn.Sequential):
                 nn.MaxPool2d(2),
             ]
         in_chans = [stem_channels] + output_channels[:-1]
-        for n_blocks, in_chan, out_chan, stride, conv, pool in zip(num_blocks, in_chans, output_channels, stage_stride,
-                                                                   stage_conv, stage_pooling):
+        for n_blocks, in_chan, out_chan, stride, conv, pool in zip(
+            num_blocks, in_chans, output_channels, stage_stride, stage_conv, stage_pooling
+        ):
             _stage = resnet_stage(in_chan, out_chan, n_blocks, stride)
             if attn_module is not None:
                 _stage.append(attn_module(out_chan))
@@ -132,17 +134,20 @@ class ResNet(nn.Sequential):
             _layers.append(nn.Sequential(*_stage))
 
         if include_top:
-            _layers.extend([
-                nn.AdaptiveAvgPool2d(1),
-                nn.Flatten(1),
-                nn.Linear(output_channels[-1], num_classes, bias=True),
-            ])
+            _layers.extend(
+                [
+                    nn.AdaptiveAvgPool2d(1),
+                    nn.Flatten(1),
+                    nn.Linear(output_channels[-1], num_classes, bias=True),
+                ]
+            )
 
         super().__init__(*_layers)
+        self.cfg = cfg
 
         for m in self.modules():
             if isinstance(m, nn.Conv2d):
-                nn.init.kaiming_normal_(m.weight, mode='fan_out', nonlinearity='relu')
+                nn.init.kaiming_normal_(m.weight, mode="fan_out", nonlinearity="relu")
             elif isinstance(m, (nn.BatchNorm2d, nn.GroupNorm)):
                 nn.init.constant_(m.weight, 1)
                 nn.init.constant_(m.bias, 0)
@@ -156,16 +161,25 @@ def _resnet(
     stage_stride: List[int],
     stage_conv: List[bool],
     stage_pooling: List[Optional[Tuple[int, int]]],
+    ignore_keys: Optional[List[str]] = None,
     **kwargs: Any,
 ) -> ResNet:
+    kwargs["num_classes"] = kwargs.get("num_classes", len(default_cfgs[arch]["classes"]))
+    kwargs["classes"] = kwargs.get("classes", default_cfgs[arch]["classes"])
 
-    kwargs['num_classes'] = kwargs.get('num_classes', len(default_cfgs[arch]['classes']))
+    _cfg = deepcopy(default_cfgs[arch])
+    _cfg["num_classes"] = kwargs["num_classes"]
+    _cfg["classes"] = kwargs["classes"]
+    kwargs.pop("classes")
 
     # Build the model
-    model = ResNet(num_blocks, output_channels, stage_stride, stage_conv, stage_pooling, **kwargs)
+    model = ResNet(num_blocks, output_channels, stage_stride, stage_conv, stage_pooling, cfg=_cfg, **kwargs)
     # Load pretrained parameters
     if pretrained:
-        load_pretrained_params(model, default_cfgs[arch]['url'])
+        # The number of classes is not the same as the number of classes in the pretrained model =>
+        # remove the last layer weights
+        _ignore_keys = ignore_keys if kwargs["num_classes"] != len(default_cfgs[arch]["classes"]) else None
+        load_pretrained_params(model, default_cfgs[arch]["url"], ignore_keys=_ignore_keys)
 
     return model
 
@@ -174,16 +188,27 @@ def _tv_resnet(
     arch: str,
     pretrained: bool,
     arch_fn,
+    ignore_keys: Optional[List[str]] = None,
     **kwargs: Any,
 ) -> TVResNet:
+    kwargs["num_classes"] = kwargs.get("num_classes", len(default_cfgs[arch]["classes"]))
+    kwargs["classes"] = kwargs.get("classes", default_cfgs[arch]["classes"])
 
-    kwargs['num_classes'] = kwargs.get('num_classes', len(default_cfgs[arch]['classes']))
+    _cfg = deepcopy(default_cfgs[arch])
+    _cfg["num_classes"] = kwargs["num_classes"]
+    _cfg["classes"] = kwargs["classes"]
+    kwargs.pop("classes")
 
     # Build the model
-    model = arch_fn(**kwargs)
+    model = arch_fn(**kwargs, weights=None)
     # Load pretrained parameters
     if pretrained:
-        load_pretrained_params(model, default_cfgs[arch]['url'])
+        # The number of classes is not the same as the number of classes in the pretrained model =>
+        # remove the last layer weights
+        _ignore_keys = ignore_keys if kwargs["num_classes"] != len(default_cfgs[arch]["classes"]) else None
+        load_pretrained_params(model, default_cfgs[arch]["url"], ignore_keys=_ignore_keys)
+
+    model.cfg = _cfg
 
     return model
 
@@ -205,7 +230,13 @@ def resnet18(pretrained: bool = False, **kwargs: Any) -> TVResNet:
         A resnet18 model
     """
 
-    return _tv_resnet('resnet18', pretrained, tv_resnet18, **kwargs)
+    return _tv_resnet(
+        "resnet18",
+        pretrained,
+        tv_resnet18,
+        ignore_keys=["fc.weight", "fc.bias"],
+        **kwargs,
+    )
 
 
 def resnet31(pretrained: bool = False, **kwargs: Any) -> ResNet:
@@ -227,7 +258,7 @@ def resnet31(pretrained: bool = False, **kwargs: Any) -> ResNet:
     """
 
     return _resnet(
-        'resnet31',
+        "resnet31",
         pretrained,
         [1, 2, 5, 3],
         [256, 256, 512, 512],
@@ -236,6 +267,7 @@ def resnet31(pretrained: bool = False, **kwargs: Any) -> ResNet:
         [(2, 2), (2, 1), None, None],
         origin_stem=False,
         stem_channels=128,
+        ignore_keys=["13.weight", "13.bias"],
         **kwargs,
     )
 
@@ -257,7 +289,13 @@ def resnet34(pretrained: bool = False, **kwargs: Any) -> TVResNet:
         A resnet34 model
     """
 
-    return _tv_resnet('resnet34', pretrained, tv_resnet34, **kwargs)
+    return _tv_resnet(
+        "resnet34",
+        pretrained,
+        tv_resnet34,
+        ignore_keys=["fc.weight", "fc.bias"],
+        **kwargs,
+    )
 
 
 def resnet34_wide(pretrained: bool = False, **kwargs: Any) -> ResNet:
@@ -278,7 +316,7 @@ def resnet34_wide(pretrained: bool = False, **kwargs: Any) -> ResNet:
     """
 
     return _resnet(
-        'resnet34_wide',
+        "resnet34_wide",
         pretrained,
         [3, 4, 6, 3],
         [128, 256, 512, 1024],
@@ -287,6 +325,7 @@ def resnet34_wide(pretrained: bool = False, **kwargs: Any) -> ResNet:
         [None] * 4,
         origin_stem=True,
         stem_channels=128,
+        ignore_keys=["10.weight", "10.bias"],
         **kwargs,
     )
 
@@ -308,4 +347,10 @@ def resnet50(pretrained: bool = False, **kwargs: Any) -> TVResNet:
         A resnet50 model
     """
 
-    return _tv_resnet('resnet50', pretrained, tv_resnet50, **kwargs)
+    return _tv_resnet(
+        "resnet50",
+        pretrained,
+        tv_resnet50,
+        ignore_keys=["fc.weight", "fc.bias"],
+        **kwargs,
+    )
