@@ -1,8 +1,9 @@
-# Copyright (C) 2021-2022, Mindee.
+# Copyright (C) 2021-2023, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
+from copy import deepcopy
 from math import ceil
 from typing import List, Optional, Tuple, Union
 
@@ -11,9 +12,20 @@ import numpy as np
 
 from .common_types import BoundingBox, Polygon4P
 
-__all__ = ['bbox_to_polygon', 'polygon_to_bbox', 'resolve_enclosing_bbox', 'resolve_enclosing_rbbox',
-           'rotate_boxes', 'compute_expanded_shape', 'rotate_image', 'estimate_page_angle',
-           'convert_to_relative_coords', 'rotate_abs_geoms']
+__all__ = [
+    "bbox_to_polygon",
+    "polygon_to_bbox",
+    "resolve_enclosing_bbox",
+    "resolve_enclosing_rbbox",
+    "rotate_boxes",
+    "compute_expanded_shape",
+    "rotate_image",
+    "estimate_page_angle",
+    "convert_to_relative_coords",
+    "rotate_abs_geoms",
+    "extract_crops",
+    "extract_rcrops",
+]
 
 
 def bbox_to_polygon(bbox: BoundingBox) -> Polygon4P:
@@ -44,23 +56,22 @@ def resolve_enclosing_bbox(bboxes: Union[List[BoundingBox], np.ndarray]) -> Unio
 
 
 def resolve_enclosing_rbbox(rbboxes: List[np.ndarray], intermed_size: int = 1024) -> np.ndarray:
-    cloud = np.concatenate(rbboxes, axis=0)
+    cloud: np.ndarray = np.concatenate(rbboxes, axis=0)
     # Convert to absolute for minAreaRect
     cloud *= intermed_size
     rect = cv2.minAreaRect(cloud.astype(np.int32))
     return cv2.boxPoints(rect) / intermed_size
 
 
-def rotate_abs_points(points: np.ndarray, angle: float = 0.) -> np.ndarray:
+def rotate_abs_points(points: np.ndarray, angle: float = 0.0) -> np.ndarray:
     """Rotate points counter-clockwise.
     Points: array of size (N, 2)
     """
 
-    angle_rad = angle * np.pi / 180.  # compute radian angle for np functions
-    rotation_mat = np.array([
-        [np.cos(angle_rad), -np.sin(angle_rad)],
-        [np.sin(angle_rad), np.cos(angle_rad)]
-    ], dtype=points.dtype)
+    angle_rad = angle * np.pi / 180.0  # compute radian angle for np functions
+    rotation_mat = np.array(
+        [[np.cos(angle_rad), -np.sin(angle_rad)], [np.sin(angle_rad), np.cos(angle_rad)]], dtype=points.dtype
+    )
     return np.matmul(points, rotation_mat.T)
 
 
@@ -75,10 +86,12 @@ def compute_expanded_shape(img_shape: Tuple[int, int], angle: float) -> Tuple[in
         the height and width of the rotated image
     """
 
-    points = np.array([
-        [img_shape[1] / 2, img_shape[0] / 2],
-        [-img_shape[1] / 2, img_shape[0] / 2],
-    ])
+    points: np.ndarray = np.array(
+        [
+            [img_shape[1] / 2, img_shape[0] / 2],
+            [-img_shape[1] / 2, img_shape[0] / 2],
+        ]
+    )
 
     rotated_points = rotate_abs_points(points, angle)
 
@@ -106,10 +119,11 @@ def rotate_abs_geoms(
     """
 
     # Switch to polygons
-    polys = np.stack(
-        [geoms[:, [0, 1]], geoms[:, [2, 1]], geoms[:, [2, 3]], geoms[:, [0, 3]]],
-        axis=1
-    ) if geoms.ndim == 2 else geoms
+    polys = (
+        np.stack([geoms[:, [0, 1]], geoms[:, [2, 1]], geoms[:, [2, 3]], geoms[:, [0, 3]]], axis=1)
+        if geoms.ndim == 2
+        else geoms
+    )
     polys = polys.astype(np.float32)
 
     # Switch to image center as referential
@@ -127,12 +141,8 @@ def rotate_abs_geoms(
     return rotated_polys
 
 
-def remap_boxes(
-    loc_preds: np.ndarray,
-    orig_shape: Tuple[int, int],
-    dest_shape: Tuple[int, int]
-) -> np.ndarray:
-    """ Remaps a batch of rotated locpred (N, 4, 2) expressed for an origin_shape to a destination_shape.
+def remap_boxes(loc_preds: np.ndarray, orig_shape: Tuple[int, int], dest_shape: Tuple[int, int]) -> np.ndarray:
+    """Remaps a batch of rotated locpred (N, 4, 2) expressed for an origin_shape to a destination_shape.
     This does not impact the absolute shape of the boxes, but allow to calculate the new relative RotatedBbox
     coordinates after a resizing of the image.
 
@@ -162,7 +172,7 @@ def rotate_boxes(
     loc_preds: np.ndarray,
     angle: float,
     orig_shape: Tuple[int, int],
-    min_angle: float = 1.,
+    min_angle: float = 1.0,
     target_shape: Optional[Tuple[int, int]] = None,
 ) -> np.ndarray:
     """Rotate a batch of straight bounding boxes (xmin, ymin, xmax, ymax, c) or rotated bounding boxes
@@ -190,22 +200,21 @@ def rotate_boxes(
                 _boxes[:, [2, 3]],
                 _boxes[:, [0, 3]],
             ],
-            axis=1
+            axis=1,
         )
     # If small angle, return boxes (no rotation)
     if abs(angle) < min_angle or abs(angle) > 90 - min_angle:
         return _boxes
     # Compute rotation matrix
-    angle_rad = angle * np.pi / 180.  # compute radian angle for np functions
-    rotation_mat = np.array([
-        [np.cos(angle_rad), -np.sin(angle_rad)],
-        [np.sin(angle_rad), np.cos(angle_rad)]
-    ], dtype=_boxes.dtype)
+    angle_rad = angle * np.pi / 180.0  # compute radian angle for np functions
+    rotation_mat = np.array(
+        [[np.cos(angle_rad), -np.sin(angle_rad)], [np.sin(angle_rad), np.cos(angle_rad)]], dtype=_boxes.dtype
+    )
     # Rotate absolute points
-    points = np.stack((_boxes[:, :, 0] * orig_shape[1], _boxes[:, :, 1] * orig_shape[0]), axis=-1)
+    points: np.ndarray = np.stack((_boxes[:, :, 0] * orig_shape[1], _boxes[:, :, 1] * orig_shape[0]), axis=-1)
     image_center = (orig_shape[1] / 2, orig_shape[0] / 2)
     rotated_points = image_center + np.matmul(points - image_center, rotation_mat)
-    rotated_boxes = np.stack(
+    rotated_boxes: np.ndarray = np.stack(
         (rotated_points[:, :, 0] / orig_shape[1], rotated_points[:, :, 1] / orig_shape[0]), axis=-1
     )
 
@@ -235,10 +244,12 @@ def rotate_image(
     """
 
     # Compute the expanded padding
+    exp_img: np.ndarray
     if expand:
-        exp_shape = compute_expanded_shape(image.shape[:-1], angle)
+        exp_shape = compute_expanded_shape(image.shape[:2], angle)  # type: ignore[arg-type]
         h_pad, w_pad = int(max(0, ceil(exp_shape[0] - image.shape[0]))), int(
-            max(0, ceil(exp_shape[1] - image.shape[1])))
+            max(0, ceil(exp_shape[1] - image.shape[1]))
+        )
         exp_img = np.pad(image, ((h_pad // 2, h_pad - h_pad // 2), (w_pad // 2, w_pad - w_pad // 2), (0, 0)))
     else:
         exp_img = image
@@ -272,9 +283,7 @@ def estimate_page_angle(polys: np.ndarray) -> float:
     yleft = polys[:, 0, 1] + polys[:, 3, 1]
     xright = polys[:, 1, 0] + polys[:, 2, 0]
     yright = polys[:, 1, 1] + polys[:, 2, 1]
-    return np.median(np.arctan(
-        (yleft - yright) / (xright - xleft)  # Y axis from top to bottom!
-    )) * 180 / np.pi
+    return float(np.median(np.arctan((yleft - yright) / (xright - xleft))) * 180 / np.pi)  # Y axis from top to bottom!
 
 
 def convert_to_relative_coords(geoms: np.ndarray, img_shape: Tuple[int, int]) -> np.ndarray:
@@ -290,14 +299,89 @@ def convert_to_relative_coords(geoms: np.ndarray, img_shape: Tuple[int, int]) ->
 
     # Polygon
     if geoms.ndim == 3 and geoms.shape[1:] == (4, 2):
-        polygons = np.empty(geoms.shape, dtype=np.float32)
+        polygons: np.ndarray = np.empty(geoms.shape, dtype=np.float32)
         polygons[..., 0] = geoms[..., 0] / img_shape[1]
         polygons[..., 1] = geoms[..., 1] / img_shape[0]
         return polygons.clip(0, 1)
     if geoms.ndim == 2 and geoms.shape[1] == 4:
-        boxes = np.empty(geoms.shape, dtype=np.float32)
+        boxes: np.ndarray = np.empty(geoms.shape, dtype=np.float32)
         boxes[:, ::2] = geoms[:, ::2] / img_shape[1]
         boxes[:, 1::2] = geoms[:, 1::2] / img_shape[0]
         return boxes.clip(0, 1)
 
     raise ValueError(f"invalid format for arg `geoms`: {geoms.shape}")
+
+
+def extract_crops(img: np.ndarray, boxes: np.ndarray, channels_last: bool = True) -> List[np.ndarray]:
+    """Created cropped images from list of bounding boxes
+    Args:
+        img: input image
+        boxes: bounding boxes of shape (N, 4) where N is the number of boxes, and the relative
+            coordinates (xmin, ymin, xmax, ymax)
+        channels_last: whether the channel dimensions is the last one instead of the last one
+    Returns:
+        list of cropped images
+    """
+    if boxes.shape[0] == 0:
+        return []
+    if boxes.shape[1] != 4:
+        raise AssertionError("boxes are expected to be relative and in order (xmin, ymin, xmax, ymax)")
+
+    # Project relative coordinates
+    _boxes = boxes.copy()
+    h, w = img.shape[:2] if channels_last else img.shape[-2:]
+    if _boxes.dtype != int:
+        _boxes[:, [0, 2]] *= w
+        _boxes[:, [1, 3]] *= h
+        _boxes = _boxes.round().astype(int)
+        # Add last index
+        _boxes[2:] += 1
+    if channels_last:
+        return deepcopy([img[box[1] : box[3], box[0] : box[2]] for box in _boxes])
+
+    return deepcopy([img[:, box[1] : box[3], box[0] : box[2]] for box in _boxes])
+
+
+def extract_rcrops(
+    img: np.ndarray, polys: np.ndarray, dtype=np.float32, channels_last: bool = True
+) -> List[np.ndarray]:
+    """Created cropped images from list of rotated bounding boxes
+    Args:
+        img: input image
+        polys: bounding boxes of shape (N, 4, 2)
+        dtype: target data type of bounding boxes
+        channels_last: whether the channel dimensions is the last one instead of the last one
+    Returns:
+        list of cropped images
+    """
+    if polys.shape[0] == 0:
+        return []
+    if polys.shape[1:] != (4, 2):
+        raise AssertionError("polys are expected to be quadrilateral, of shape (N, 4, 2)")
+
+    # Project relative coordinates
+    _boxes = polys.copy()
+    height, width = img.shape[:2] if channels_last else img.shape[-2:]
+    if _boxes.dtype != int:
+        _boxes[:, :, 0] *= width
+        _boxes[:, :, 1] *= height
+
+    src_pts = _boxes[:, :3].astype(np.float32)
+    # Preserve size
+    d1 = np.linalg.norm(src_pts[:, 0] - src_pts[:, 1], axis=-1)
+    d2 = np.linalg.norm(src_pts[:, 1] - src_pts[:, 2], axis=-1)
+    # (N, 3, 2)
+    dst_pts = np.zeros((_boxes.shape[0], 3, 2), dtype=dtype)
+    dst_pts[:, 1, 0] = dst_pts[:, 2, 0] = d1 - 1
+    dst_pts[:, 2, 1] = d2 - 1
+    # Use a warp transformation to extract the crop
+    crops = [
+        cv2.warpAffine(
+            img if channels_last else img.transpose(1, 2, 0),
+            # Transformation matrix
+            cv2.getAffineTransform(src_pts[idx], dst_pts[idx]),
+            (int(d1[idx]), int(d2[idx])),
+        )
+        for idx in range(_boxes.shape[0])
+    ]
+    return crops
